from imgnet.imgnet_utils.common_utils import read_yaml
import os
import yaml
import logging
import pandas as pd
from imgnet.imgnet_tools.all_encryption import decodeFile

def trainEngine(request_data,params_path, config_path):
    print("ok")
    config = read_yaml(config_path)

    paths = config["data_path"]
    print(paths)
    IMGNET_ENGINE = paths["IMGNET_ENGINE"]
    ARTIFACTS = paths["ARTIFACTS"]
    SECTION = paths["SECTION"]

    save_data = os.path.join(IMGNET_ENGINE, ARTIFACTS, SECTION)

    try:
        if request_data:
            print(request_data)
            # if 'model_obj' in request_data:
            #     MODEL_NAME = request_data['model_obj']
            #
            # if 'batch_size' in request_data:
            #     BATCH_SIZE = int(request_data['batch_size'])
            #
            # if 'augmentation' in request_data:
            #     AUGMENTATION = request_data['augmentation']
            #
            # if 'optimizer' in request_data:
            #     OPTIMIZER = request_data['optimizer']
            #
            # if 'no_of_epochs' in request_data:
            #     EPOCHS = int(request_data['no_of_epochs'])
            #
            # if 'loss_func' in request_data:
            #     LOSS_FUNC = request_data['loss_func']
            #
            # fileName = request_data['files'][0]['name']
            # # print(f"name of file: {fileName}")
            # codedvalue = request_data['files'][0]['base64']
            # codedvalue = codedvalue.split(',')
            # # os_operation.remove_folder(os.path.join(save_data,'data'))
            #
            # codedvalue = codedvalue[-1]
            # decodeFile(codedvalue, os.path.join(save_data, fileName))
            #
            # config['AUGMENTATION'] = AUGMENTATION
            # config['BATCH_SIZE'] = BATCH_SIZE
            # config['MODEL_NAME'] = MODEL_NAME
            # config['OPTIMIZER'] = OPTIMIZER
            # config['EPOCHS'] = EPOCHS
            # config['LOSS_FUNC'] = LOSS_FUNC
            # print(config)
            #
            # with open('params/classification.yaml', 'w') as f:
            #     print("Putting file inside classification,yaml")
            #     yaml.dump(config, f)

            return "Model Successfully trained"

    except Exception as e:
        return f"Somthing went wrong! {e}"




import json

import yaml
from flask import Flask, render_template, request,jsonify
from flask_cors import CORS, cross_origin
from imgnet_engine.classification.imgnet_classification import trainEngine
#initializing flask app
from threading import Timer
import webbrowser
app=Flask(__name__)
CORS(app)

config_path="imgnet_config/classification.yaml"

@app.route('/',methods=['GET'])  # route to display the home page
def homePage():
    return render_template("home.html")

@app.route('/classificationInput',methods=['GET','POST'])
def classificationInput():
    '''
      Description: Here let's take input for image classification
    '''
    return render_template("classificationInput.html")


@app.route('/trainModel',methods=['GET','POST'])
def trainModel():
    '''
      Description: Here let's take input for image classification
    '''
    try:
        if request.method == 'POST':
            try:
                # Data config
                TRAINING_DATA_DIR = request.form['TRAINING_DATA_DIR']
                AUGMENTATION = request.form['AUGMENTATION']
                BATCHSIZE = int(request.form['BATCHSIZE'])

                # Model config
                MODEL_NAME = request.form['MODEL_NAME']
                FREEZE_ALL_LAYER = request.form['FREEZE_ALL_LAYER']
                OPTIMIZER = request.form['OPTIMIZER']
                EPOCHS = int(request.form['EPOCHS'])
                LOSSFUN = request.form['LOSSFUN']

                config = {
                    "TRAIN_DATA_DIR": TRAINING_DATA_DIR,
                    "AUGMENTATION": AUGMENTATION,
                    "BATCHSIZE": BATCHSIZE,
                    "MODEL_NAME": MODEL_NAME,
                    "EPOCHS": EPOCHS,
                    "FREEZE_ALL": FREEZE_ALL_LAYER,
                    "OPTIMIZER": OPTIMIZER,
                    "LOSS_FUNC": LOSSFUN

                }

                with open('imgnet_config/classification.yaml', 'w') as f:
                    print("Putting file inside classification,yaml")
                    yaml.dump(config, f)


                hist = trainEngine(config_path)

                return render_template('input_form.html', output=hist)

            except Exception as e:
                print('The Exception message is: ', e)
                return 'something is wrong'

    except Exception as e:
        print(e)

@app.route('/predictModel',methods=['GET','POST'])
def predictModel():
    '''
      Description: Here let's take input for image classification
    '''
    if request.method=='POST':
        print("Method called")
        data=request.form['MODEL_NAME']
        print(data)
        data=jsonify(data)
    else:
        return render_template("classificationInput.html")


def open_browser():
    webbrowser.open_new('http://127.0.0.1:8080/')

def start_app():
    Timer(1, open_browser).start()
    app.run(host="127.0.0.1", port=8080,debug=True)



if __name__ == "__main__":
    start_app()




import base64


def decodeFile(fileString, filePath):
    """Decode an image from a base64 string and write it to a file.
    Args:
        imgstring ([string]): [Image path to be decode]
        fileName ([string]): [Name of the file]
    """
    print(filePath)
    zipFile = base64.b64decode(fileString)
    # return zipFile
    with open(filePath, 'wb') as f:
        f.write(zipFile)
        f.close()
        print("file closed")


def encodeImageIntoBase64(croppedImagePath):
    """Decode an image from a base64 string and write it to a file.
    Args:
        croppedImagePath ([string]): [Image path to be decode]

    """
    with open(croppedImagePath, "rb") as f:
        return base64.b64encode(f.read())


def encodeModel(modelPath):
    """Encode a file from base64 and return it

    Args:
        modelPath ([String]): [Model path to be encode]
    """
    with open(modelPath, "rb") as f:
        return base64.b64decode(f.read())